% create high density of points for buds that are 1 radius above a SLB
% RECREATE FOR NANOPARTICLES - 170216

clear all
clc
subplot(1,1,1)
hold off
plot(1,1)
addpath('./functions')

%% INPUTS
density_parameter = 4; %num/nm^2;  4 --> 10^6 if flat
sur_box_side = 100;
top_rad = 50 ;
bot_rad = 20;
res = 10;

fold_to_save = ['vary_bud_height_' num2str(top_rad) '_' num2str(density_parameter) '_' num2str(sur_box_side) '\topology\'];
mkdir(fold_to_save)

%% loop over heights
for height = 0:5:140


clear x_con x_top x_slb x x_all 

bz = (height-top_rad); % absolute height of top center
v  = abs(bz-bot_rad); % vertical edge of triangle
hyp = top_rad+bot_rad; % hypotenuse of triangle
bot_sph_r = sqrt(hyp^2-v^2); % radial distance to bot center
th1 = acosd( v/hyp ); % top angle in triangle
th2 = 90-th1; % bottom angle in triange
if bz>=bot_rad % bz always positive here
    th_top = 180-th1; % from z axis to intersect line
    th_bot = 90-th2; % from z axis to intersect line
    inter_z = bz-top_rad*cosd(th1);
    area_top = 4*pi*top_rad^2-2*pi*top_rad*(top_rad-(bz-inter_z)); % http://mathworld.wolfram.com/SphericalCap.html
else  % top center under bottom center
    th_top = 90-th2; % from z axis to intersect line
    th_bot = 180-th1; % from z axis to intersect line
    inter_z = bot_rad-bot_rad*cosd(th1);
    area_top = 2*pi*top_rad*(height-inter_z);
end
th_botp = 180-th_bot; %from -zaxis to inter-line in outer sphere

area_slb = (2*sur_box_side)^2-pi*bot_sph_r^2;

dth = 10^-5;
th  = 0:dth:th_botp;
rn = bot_sph_r-bot_rad*sind(th);
area_con = sum(dth*pi/180*bot_rad * 2*pi*rn); % con = connection

area_tot = area_top+area_slb+area_con;

x_slb = (rand(round(density_parameter*4*sur_box_side^2),2)*2-1)*sur_box_side;
r = sqrt(x_slb(:,1).^2+x_slb(:,2).^2);
x_slb = x_slb(r>=bot_sph_r,:);
x_slb(:,3) = 0;

if top_rad > 0
    v_box = (2*top_rad)^3; % used
    v_sph = 4/3*pi*top_rad^3;
    a_sph = 4*pi*top_rad^2;
    x_top = rand(round(density_parameter*a_sph*v_box/v_sph),3)*2-1;
    r = x_top(:,1).^2+x_top(:,2).^2+x_top(:,3).^2;
    x_top = x_top(r<=1,:);
    r = sqrt(x_top(:,1).^2+x_top(:,2).^2+x_top(:,3).^2);
    x_top(:,1) = x_top(:,1)./r;
    x_top(:,2) = x_top(:,2)./r;
    x_top(:,3) = x_top(:,3)./r;
    x_top = x_top*top_rad;
    x_top(:,3) = x_top(:,3)-max(x_top(:,3))+height;
    x_top = x_top(x_top(:,3)>inter_z,:);

    num_con = round(area_con*density_parameter);
    x_con = zeros(num_con,3);
    
    % aim for 20 per loop, 
    meanr = mean([top_rad bot_sph_r]);
    dth = 20/density_parameter/2/pi/meanr/bot_rad/pi*180; % arbitrary?
    max_th = th_botp;
    sofar = 0;
    
    subplot(1,1,1); hold off; plot(bot_sph_r-bot_rad*sind(0:1:180))
    pause(0.5)
    if min(bot_sph_r-bot_rad*sind(0:1:max_th))<0
        disp('fission!')
        disp(height)
        return;
    end
        
    
    while sofar < num_con
        th = rand*max_th;
        z = bot_rad-bot_rad*cosd(th);
        r = bot_sph_r-bot_rad*sind(th);
        

        
%         if bot_sph_r>bot_rad
%             disp('fission!')
%             disp(height)
%             return
%         end
%         disp(r)
        
        num = round(density_parameter*2*pi*r*dth*bot_rad*pi/180);
        ph = rand(num,1)*360;
        x_con(sofar+(1:num),1) = r*cosd(ph);
        x_con(sofar+(1:num),2) = r*sind(ph);
        x_con(sofar+(1:num),3) = z;
        sofar = sofar+num;
    end
    x_con = x_con(1:sofar,:);
else
    x_con(1,1:3) = NaN;
    x_top(1,1:3) = NaN;
end

disp(['density:    ' num2str([size(x_top,1)/area_top size(x_con,1)/area_con size(x_slb,1)/area_slb],3)])
x = [x_slb; x_top; x_con];
r = sqrt(x(:,1).^2+x(:,2).^2);
max_nzr =sur_box_side;
x = x(r < max_nzr,:);

subplot(1,1,1)
hold off
plot(1,1)
subplot(3,1,1:2)
plot3(x(1:res:end,1),x(1:res:end,2),x(1:res:end,3),'.')
axis equal tight vis3d
title(num2str([height top_rad bot_rad density_parameter]))

subplot(3,1,3)
plot3(x(1:res:end,1),x(1:res:end,2),x(1:res:end,3),'.')
axis equal tight vis3d
view([0 0])
title(['num points: ' num2str(size(x))])

saveas(gcf,[fold_to_save 'MemTopology_' num2str(height) '_' num2str(top_rad) '_' num2str(bot_rad) '_' num2str(density_parameter) '.jpg'],'jpg')
save([fold_to_save 'MemTopology_' num2str(height) '_' num2str(top_rad) '_' num2str(bot_rad) '_' num2str(density_parameter) '.mat'],'x','density_parameter','top_rad','bot_rad','height')

end

